#!/bin/sh -

# Halt on first error and show what's happening

set -ex

# Generate a pile of config files, keys, and PKCS #10 requests

for i in TA POKE-EE RPKI IRDB IRBE SELF-1
do

  case $i in
    TA|SELF-1)
      ca=true
      touch ISC-$i.idx
      echo >ISC-$i.cnm 01
      ;;
    *)
      ca=false
      ;;
  esac

  test -r ISC-$i.cnf || cat >ISC-$i.cnf <<-EOF

	[ req ]
	distinguished_name	= req_dn
	x509_extensions		= req_x509_ext
	prompt			= no
	default_md		= sha256

	[ req_dn ]
	CN			= ISC $i Test Certificate

	[ req_x509_ext ]
	basicConstraints	= CA:$ca
	subjectKeyIdentifier	= hash
	authorityKeyIdentifier	= keyid:always

	[ ca ]
	default_ca = ca_default

	[ ca_default ]

	certificate             = ISC-$i.cer
	serial                  = ISC-$i.srl
	private_key             = ISC-$i.key
	database                = ISC-$i.idx
	crlnumber               = ISC-$i.cnm
	default_crl_days        = 30
	default_md              = sha256

	EOF

  test -r ISC-$i.key ||
  openssl genrsa -out ISC-$i.key 2048

  test -r ISC-$i.req ||
  openssl req -new -sha256 -key ISC-$i.key -out ISC-$i.req -config ISC-$i.cnf

done

# Generate the BPKI TA

test -r ISC-TA.cer ||
openssl x509 -sha256 -req -in ISC-TA.req -out ISC-TA.cer	\
    -extfile ISC-TA.cnf -extensions req_x509_ext		\
    -signkey ISC-TA.key -days 60

# Sign children of the BPKI TA

for i in RPKI IRDB IRBE SELF-1
do
  test -r ISC-$i.cer ||
  openssl x509 -sha256 -req -in ISC-$i.req -out ISC-$i.cer	\
      -extfile ISC-$i.cnf -extensions req_x509_ext		\
      -CA ISC-TA.cer -CAkey ISC-TA.key -CAcreateserial
done

for i in TA SELF-1
do
  openssl ca -batch -gencrl -out ISC-$i.crl -config ISC-$i.cnf
done

# Need to cross-certify:
# 
#   APNIC-CMS-CERT   into ISC-SELF-1's BPKI (ISC-SELF-1-APNIC-CMS-CERT.cer)
#   APNIC-HTTPS-CERT into ISC-SELF-1's BPKI (ISC-SELF-1-APNIC-HTTPS-CERT.cer)
#
#   ISC-SELF-1       into APNIC-CMS-CERT's BPKI (APNIC-CMS-CERT-ISC-SELF-1.cer)
#
# Except that the latter is APNIC's problem and what we need to do is send
# them our BSC .req after we get it from irbe-cli.py
#
# So at this point the next thing we need to do is start up irdbd.py
# at least for long enough to create the BSC and get a .req out of it.

# Blow away old rpkid database.
#
mysql -u rpki -p`awk '$1 == "sql-password" {print $3}' rpkid.conf` \
    rpki <rpkid.sql

# Start rpkid so we can configure it, make sure we shut it down on exit
# If we're running under screen, just run it in a different screen instead.

if test -n "$STY"
then
  screen python rpkid.py
else
  python rpkid.py >>rpkid.log 2>&1 & rpkid=$!
  trap "kill $rpkid" 0 1 2 3 13 15
fi

# Create a self instance

sleep 15

python irbe-cli.py self --action create --crl_interval 84600 --regen_margin 42300 \
    --bpki_cert ISC-SELF-1.cer

# Create a business signing context and generate new PKCS #10 request

python irbe-cli.py --pem_out bsc.req bsc --action create --self_id 1 \
    --generate_keypair

# As it happened, APNIC's test environment couldn't accept PKCS #10 after all,
# so I had to generate certs.  Goofed up first time, what finally worked was:
#
# for i in POKE-EE bsc
#   do
#     openssl x509 -sha256 -req -in ISC-$i.req -out ISC-$i.cer \
#     -extfile ISC-POKE-EE.cnf -extensions req_x509_ext        \
#     -CA ISC-SELF-1.cer -CAkey ISC-SELF-1.key -CAcreateserial
#   done
#
# ... because config files generate above are a little squirrely.
#
# But this worked well enough to use both ISC-POKE-EE.cer and ISC-bsc.cer
# as client certs talking to APNIC, which was good enough for the moment.
